////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.preference.PreferenceManager;


/***********************************************************************************************
A singleton class for storing and retrieving application settings
***********************************************************************************************/
public class Settings {

  /** The application <i>Context</i> for which the settings are read and written **/
  static Context application = null;
  /** Read access to the <i>application</i> settings **/
  static SharedPreferences reader = null;
  /** Write access to the <i>application</i> settings **/
  static Editor writer = null;


  /***********************************************************************************************
  Gets the <code>String</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>String</code>
  ***********************************************************************************************/
  public static String get(String code){
    return getString(code);
  }


  /***********************************************************************************************
  Gets the <i>application</i> for which the settings are read and written
  @return the <i>application</i> <code>Activity</code>
  ***********************************************************************************************/
  public static Context getApplication(){
    return application;
  }


  /***********************************************************************************************
   Gets the <code>boolean</code> value for the <code>code</code> setting
   @param code the setting to get the value for
   @return the setting value as a <code>boolean</code>
   ***********************************************************************************************/
  public static boolean getBoolean(String code){
    return reader.getBoolean(code,false);
  }


  /***********************************************************************************************
  Gets the <code>byte</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>byte</code>
  ***********************************************************************************************/
  public static byte getByte(String code){
    return (byte)reader.getInt(code,-1);
  }


  /***********************************************************************************************
  Gets the <code>char</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>char</code>
  ***********************************************************************************************/
  public static char getChar(String code){
    String value = reader.getString(code,"");
    if (value.equals("")) {
      return (char)0;
    }
    return value.charAt(0);
  }


  /***********************************************************************************************
  Gets the <code>double</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>double</code>
  ***********************************************************************************************/
  public static double getDouble(String code){
    return reader.getFloat(code,-1);
  }


  /***********************************************************************************************
  Gets the <code>float</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>float</code>
  ***********************************************************************************************/
  public static float getFloat(String code){
    return reader.getFloat(code,-1);
  }


  /***********************************************************************************************
  Gets the <code>int</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>int</code>
  ***********************************************************************************************/
  public static int getInt(String code){
    return reader.getInt(code,-1);
  }


  /***********************************************************************************************
  Gets the <code>long</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>long</code>
  ***********************************************************************************************/
  public static long getLong(String code){
    return reader.getLong(code,-1);
  }


  /***********************************************************************************************
  Gets the <code>short</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>short</code>
  ***********************************************************************************************/
  public static short getShort(String code){
    return (short)reader.getInt(code,-1);
  }


  /***********************************************************************************************
  Gets the <code>String</code> value for the <code>code</code> setting
  @param code the setting to get the value for
  @return the setting value as a <code>String</code>
  ***********************************************************************************************/
  public static String getString(String code){
    return reader.getString(code,"");
  }


  /***********************************************************************************************
  Gets whether the <i>application</i> for which the settings are read and written has been set
  @return <b>true</b> if the <i>application</i> <code>Activity</code> is set<br />
          <b>false</b> otherwise
  ***********************************************************************************************/
  public static boolean hasApplication(){
    return (application != null);
  }


  /***********************************************************************************************
  Sets the <code>boolean</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>boolean</code>
  ***********************************************************************************************/
  public static void set(String code,boolean value){
    writer.putBoolean(code,value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>byte</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>byte</code>
  ***********************************************************************************************/
  public static void set(String code,byte value){
    writer.putInt(code,value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>char</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>char</code>
  ***********************************************************************************************/
  public static void set(String code,char value){
    writer.putString(code,"" + value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>double</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>double</code>
  ***********************************************************************************************/
  public static void set(String code,double value){
    writer.putFloat(code,(float)value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>float</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>float</code>
  ***********************************************************************************************/
  public static void set(String code,float value){
    writer.putFloat(code,value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>int</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as an <code>int</code>
  ***********************************************************************************************/
  public static void set(String code,int value){
    writer.putInt(code,value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>long</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>long</code>
  ***********************************************************************************************/
  public static void set(String code,long value){
    writer.putLong(code,value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>short</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>short</code>
  ***********************************************************************************************/
  public static void set(String code,short value){
    writer.putInt(code,value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <code>String</code> <code>value</code> of the setting <code>code</code>
  @param code the setting to set the <code>value</code> of
  @param value the new value of the setting, as a <code>String</code>
  ***********************************************************************************************/
  public static void set(String code,String value){
    writer.putString(code,value);
    writer.commit();
  }


  /***********************************************************************************************
  Sets the <i>application</i> for which the settings are read and written, and initialises the
  <i>reader</i> and <i>writer</i> global variables
  @param activity the <i>application</i> <code>Activity</code>
  ***********************************************************************************************/
  public static void setApplication(Context context){
    application = context;
    reader = PreferenceManager.getDefaultSharedPreferences(application);
    writer = reader.edit();
  }

}