////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android;

import org.watto.android.event.WSPreferenceClickableInterface;
import org.watto.android.event.listener.WSPreferenceClickableListener;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.PreferenceActivity;


/***********************************************************************************************
Shows the screen that allows the user to change the program settings
***********************************************************************************************/
public class WSSettingsChanger extends PreferenceActivity implements WSPreferenceClickableInterface {

  /***********************************************************************************************
  Creates the <code>Activity</code> from the XML file
  @param savedInstanceState the saved state of the <code>Activity</code>
  ***********************************************************************************************/
  public void onCreate(Bundle savedInstanceState){
    super.onCreate(savedInstanceState);

    // Creates the view from the XML file
    addPreferencesFromResource(R.layout.settings);
    
    // Put listeners on the Settings that need to be actioned directly
    WSPreferenceClickableListener clickListener = new WSPreferenceClickableListener(this);
    
    Preference preference;
    
    preference = findPreference("DonateUrl");
    if (preference != null){
      preference.setOnPreferenceClickListener(clickListener);
    }
    
    preference = findPreference("WebsiteUrl");
    if (preference != null){
      preference.setOnPreferenceClickListener(clickListener);
    }
  }
  
  
  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>Preference</code>
  @param preference the <code>Preference</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(Preference preference){
    String key = preference.getKey();
    if (key.equals("DonateUrl")) {
      Intent intent = new Intent(Intent.ACTION_VIEW,Uri.parse(Language.get(R.string.donate_url)));
      startActivity(intent);
    }
    else if (key.equals("WebsiteUrl")) {
      Intent intent = new Intent(Intent.ACTION_VIEW,Uri.parse(Language.get(R.string.website_url)));
      startActivity(intent);
    }
    else {
      return false;
    }
    return true;
  }
  
  


}
