////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.component;

import org.watto.android.event.WSViewClickableInterface;
import org.watto.android.event.listener.WSViewClickableListener;
import android.view.View;
import android.widget.RadioButton;


/***********************************************************************************************
Defines a group that holds <code>RadioButton</code>s and ensures only 1 radio button is checked
at a time.
***********************************************************************************************/
public class ButtonGroup implements WSViewClickableInterface {

  /** The RadioButtons that belong to the group **/
  RadioButton[] radioButtons;


  /***********************************************************************************************
  Creates the group for a number of <code>RadioButton</code>s
  @param radioButtons the <code>RadioButton</code>s that form this group
   ***********************************************************************************************/
  public ButtonGroup(RadioButton ... radioButtons){
    this.radioButtons = radioButtons;

    WSViewClickableListener viewClickableListener = new WSViewClickableListener(this);

    int radioCount = radioButtons.length;
    for (int i = 0;i < radioCount;i++) {
      radioButtons[i].setOnClickListener(viewClickableListener);
    }
  }


  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    if (!(source instanceof WSRadioButton)) {
      return false;
    }

    int radioCount = radioButtons.length;
    for (int i = 0;i < radioCount;i++) {
      RadioButton radioButton = radioButtons[i];
      if (source == radioButton) {
        radioButton.setChecked(true);
      }
      else {
        radioButton.setChecked(false);
      }
    }
    return true;
  }

}
