////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.component;

import org.watto.android.R;
import org.watto.android.event.WSDialogClickableInterface;
import org.watto.android.event.listener.WSDialogClickableListener;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;


/***********************************************************************************************
Helper methods for generating and showing generic <code>Dialog</code>s
***********************************************************************************************/
public class WSAndroidPopup {

  /***********************************************************************************************
  Shows an error message to the user in a <code>Dialog</code>
  @param context the <code>Context</code> in which to show the error
  @param stringID the ID of the error message in the <code>strings.xml</code> resource
  ***********************************************************************************************/
  public static void showError(Context context,int stringID){
    AlertDialog.Builder builder = new AlertDialog.Builder(context);

    builder.setMessage(context.getResources().getString(stringID));
    builder.setCancelable(false);
    builder.setPositiveButton(context.getResources().getString(R.string.ok),new DialogInterface.OnClickListener() {

      public void onClick(DialogInterface dialog,int id){
        dialog.cancel();
      }
    });

    AlertDialog alert = builder.create();
    alert.show();

  }


  /***********************************************************************************************
  Shows an information message to the user in a <code>Dialog</code>
  @param context the <code>Context</code> in which to show the information
  @param stringID the ID of the information message in the <code>strings.xml</code> resource
  ***********************************************************************************************/
  public static void showInformation(Context context,int stringID){
    showError(context,stringID);
  }


  /***********************************************************************************************
  Shows a question/confirmation message to the user in a <code>Dialog</code>
  @param context the <code>Context</code> in which to show the question
  @param stringID the ID of the error message in the <code>strings.xml</code> resource
  ***********************************************************************************************/
  public static AlertDialog showQuestion(Context context,int stringID){
    AlertDialog.Builder builder = new AlertDialog.Builder(context);

    WSDialogClickableListener listener;
    if (context instanceof WSDialogClickableInterface) {
      listener = new WSDialogClickableListener((WSDialogClickableInterface)context);
    }
    else {
      return null;
    }

    builder.setMessage(context.getResources().getString(stringID));
    builder.setCancelable(false);
    builder.setPositiveButton(context.getResources().getString(R.string.yes),listener);
    builder.setNegativeButton(context.getResources().getString(R.string.no),listener);

    AlertDialog alert = builder.create();
    alert.show();

    return alert;

  }

}
