////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.component;

import org.watto.android.R;
import org.watto.android.Settings;
import org.watto.android.event.WSDialogClickableInterface;
import org.watto.android.event.WSViewClickableInterface;
import org.watto.android.event.listener.WSDialogClickableListener;
import org.watto.android.event.listener.WSViewClickableListener;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.content.res.XmlResourceParser;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;


/***********************************************************************************************
An extension of <code>TextView</code> that displays a <code>String</code> and, upon clicking the
<code>TextView</code>, displays a popup allowing the user to choose a value from a list selection.
***********************************************************************************************/
public class WSRadioButtonDropdown extends LinearLayout implements WSAndroidComponent, WSViewClickableInterface, WSDialogClickableInterface {

  WSExtraInfo extraInfo;

  /** The titles to display on the list **/
  String[] titlesArray = new String[0];
  /** The key values for each item in the list **/
  String[] valuesArray = new String[0];
  /** The selected item in the list **/
  String selectedValue = "";
  /** The setting that remembers the value of this field **/
  String setting = null;

  /** The listener that is triggered when the <i>selectedValue</i> is changed, so you can react to a change in value **/
  WSViewClickableListener itemChangedListener = null;


  /***********************************************************************************************
  Constructor
  @param context the <code>Context</code> holding the <code>WSTextView</code>
  @param attributes the properties of this <code>WSTextView</code>, from the XML file
  ***********************************************************************************************/
  public WSRadioButtonDropdown(Context context, AttributeSet attributes){
    super(context,attributes);
    extraInfo = new WSExtraInfo(this);

    scanAttributes(attributes);

    // Populate the globals, if they've been specified in the XML file
    Resources resources = getResources();
    String namespace = null;
    
    String hint = null;
    
    if (extraInfo.has("namespace")) {
      namespace = (String)extraInfo.get("namespace");
    }
    if (extraInfo.has("titlesArray")) {
      int resourceId = resources.getIdentifier(((String)extraInfo.get("titlesArray")).substring(1),null,namespace);
      if (resourceId != 0) {
        titlesArray = resources.getStringArray(resourceId);
      }
    }
    if (extraInfo.has("valuesArray")) {
      int resourceId = resources.getIdentifier(((String)extraInfo.get("valuesArray")).substring(1),null,namespace);
      if (resourceId != 0) {
        valuesArray = resources.getStringArray(resourceId);
      }
    }
    if (extraInfo.has("selectedValue")) {
      selectedValue = (String)extraInfo.get("selectedValue");
    }
    if (extraInfo.has("setting")) {
      setting = (String)extraInfo.get("setting");
    }
    if (extraInfo.has("hint")) {
      int resourceId = resources.getIdentifier(((String)extraInfo.get("hint")).substring(1),null,namespace);
      if (resourceId != 0) {
        hint = resources.getString(resourceId);
      }
    }

    LayoutInflater inflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    inflater.inflate(R.layout.wsradiobuttondropdown,this);

    // Set the click listeners
    WSViewClickableListener listener = new WSViewClickableListener(this);

    TextView currentValueField = (TextView)findViewById(R.id.wsradiobuttondropdown_tf_currentvalue);
    currentValueField.setOnClickListener(listener);
    // set the hint on the text field
    currentValueField.setHint(hint);
    
    // Set the selected value to be the value saved in the Settings
    if (setting != null && Settings.hasApplication()){
      setSelectedValue(Settings.getString(setting));
    }

    Button arrowButton = (Button)findViewById(R.id.wsradiobuttondropdown_button_arrow);
    arrowButton.setOnClickListener(listener);

    // Sets the text on the currentValueField with the selected value title
    setSelectedValue(selectedValue);

  }


  /***********************************************************************************************
  Adds a single item of extra information
  @param key the key to identify the <i>value</i>
  @param value the <code>Object</code>
  ***********************************************************************************************/
  public void addExtraInfo(String key,Object value){
    extraInfo.add(key,value);
  }


  /***********************************************************************************************
  Triggers the onClick() of the <i>itemChangedListener</i> when the <i>selectedValue</i> has changed
  ***********************************************************************************************/
  private void fireItemChangedEvent(){
    if (itemChangedListener != null) {
      itemChangedListener.onClick(this);
    }
  }


  /***********************************************************************************************
  Gets the additional information associated with this <code>View</code>. Used when triggering a
  <code>ContextMenu</code> from this component.
  @return any additional information attached to this <code>View</code>
  ***********************************************************************************************/
  public ContextMenuInfo getContextMenuInfo(){
    return extraInfo;
  }


  /***********************************************************************************************
  Gets the additional information associated with this <code>View</code>.
  @return any additional information attached to this <code>View</code>
  ***********************************************************************************************/
  public WSExtraInfo getExtraInfo(){
    return extraInfo;
  }


  /***********************************************************************************************
  Gets the <code>Object</code> with the given <i>key</i>
  @param key the key of the <code>Object</code> to return
  @return the <code>Object</code> for the <i>key</i>, or <b>null</b> if the key doesn't exist
  ***********************************************************************************************/
  public Object getExtraInfo(String key){
    return extraInfo.get(key);
  }


  /***********************************************************************************************
  Gets the index in the <i>valuesArray</i> of the <i>selectedValue</i>
  @return the index of the <i>selectedValue</i> in the <i>valuesArray</i>, or <b>-1</b> if the value
          doesn't exist in the <i>valuesArray</i>
  ***********************************************************************************************/
  public int getSelectedIndex(){
    int valueCount = valuesArray.length;
    for (int i = 0;i < valueCount;i++) {
      if (valuesArray[i].equals(selectedValue)) {
        return i;
      }
    }
    return -1;
  }


  /***********************************************************************************************
  Gets the <i>selectedValue</i>
  @return the <i>selectedValue</i>
  ***********************************************************************************************/
  public String getSelectedValue(){
    return selectedValue;
  }


  /***********************************************************************************************
  Gets the <i>selectedValue</i> title
  @return the <i>selectedValue</i> title
  ***********************************************************************************************/
  public String getSelectedValueTitle(){
    int selectedIndex = getSelectedIndex();
    if (selectedIndex == -1) {
      return "";
    }
    return titlesArray[selectedIndex];
  }


  /***********************************************************************************************
  Determines whether there is extra information stored for the given <i>key</i>
  @param key the key of the <code>Object</code> to find
  @return <b>true</b> if extra information for the <i>key</i> exists <br />
          <b>false</b> if extra information for the key is not defined
  ***********************************************************************************************/
  public boolean hasExtraInfo(String key){
    return extraInfo.has(key);
  }


  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>DialogInterface</code> that triggered the event
  @param button the <code>Button</code> that was clicked on the dialog
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(DialogInterface dialog,int button){
    AlertDialog alert = (AlertDialog)dialog;
    int selectedItem = alert.getListView().getCheckedItemPosition();
    setSelectedValue(valuesArray[selectedItem]);
    alert.dismiss();

    fireItemChangedEvent();
    return true;
  }


  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    int id = source.getId();

    if (id == R.id.wsradiobuttondropdown_tf_currentvalue || id == R.id.wsradiobuttondropdown_button_arrow) {
      // show the popup

      Resources resources = getResources();

      // Set the title
      String title = "";
      if (extraInfo.has("title")) {
        title = resources.getString(resources.getIdentifier((String)extraInfo.get("title"),null,null));
      }
      else {
        resources.getString(R.string.dropdown_title);
      }

      int selectedIndex = getSelectedIndex();

      // build and show the dialog
      AlertDialog.Builder builder = new AlertDialog.Builder(this.getContext());
      builder.setTitle(title);
      builder.setSingleChoiceItems(titlesArray,selectedIndex,new WSDialogClickableListener(this));
      AlertDialog alert = builder.create();
      alert.show();
      return true;
    }

    return false;
  }


  /***********************************************************************************************
  Removes a single item of extra information
  @param key the key to identify the <i>value</i> to remove
  ***********************************************************************************************/
  public void removeExtraInfo(String key){
    extraInfo.remove(key);
  }


  /***********************************************************************************************
  Scans the <code>AttributeSet</code> for attributes in the <i>watto</i> namespace, then sets them
  in the <i>extraInfo</i> 
  @param attributes the properties of this <code>WSTextView</code>, from the XML file
  ***********************************************************************************************/
  public void scanAttributes(AttributeSet attributes){
    if (attributes instanceof XmlResourceParser) {
      XmlResourceParser xmlAttributes = (XmlResourceParser)attributes;

      String namespace = "http://www.watto.org/android/schema";

      int attributeCount = xmlAttributes.getAttributeCount();
      for (int i = 0;i < attributeCount;i++) {
        String prefix = xmlAttributes.getAttributeNamespace(i);
        if (prefix != null && prefix.equals(namespace)) {
          String name = xmlAttributes.getAttributeName(i);
          String value = xmlAttributes.getAttributeValue(i);
          extraInfo.add(name,value);
        }
      }
    }
  }


  /***********************************************************************************************
  Sets the additional information associated with this <code>View</code>.
  @param extraInfo any additional information for this <code>View</code>
  ***********************************************************************************************/
  public void setExtraInfo(WSExtraInfo extraInfo){
    this.extraInfo = extraInfo;
  }


  /***********************************************************************************************
  Sets the <code>WSViewClickableListener</code> to trigger when the <i>selectedValue</i> is changed
  @param itemChangedListener The <code>WSViewClickableListener</code> to trigger
  ***********************************************************************************************/
  public void setItemChangedListener(WSViewClickableListener itemChangedListener){
    this.itemChangedListener = itemChangedListener;
  }



  /***********************************************************************************************
  Sets the <i>selectedValue</i> in the <i>valuesArray</i>
  @param selectedValue the <i>selectedValue</i> in the <i>valuesArray</i>
  ***********************************************************************************************/
  public void setSelectedValue(String selectedValue){
    this.selectedValue = selectedValue;

    TextView currentValueField = (TextView)findViewById(R.id.wsradiobuttondropdown_tf_currentvalue);
    currentValueField.setText(getSelectedValueTitle());
    
    if (setting != null && Settings.hasApplication()){
      // remember the selected value in the Settings
      Settings.set(setting,selectedValue);
    }
  }


  /***********************************************************************************************
  Sets the titles to show in the list
  @param titlesArray the titles to show in the list
  ***********************************************************************************************/
  public void setTitles(String[] titlesArray){
    this.titlesArray = titlesArray;
  }


  /***********************************************************************************************
  Sets the key values for each item in the list
  @param valuesArray the key values for each item in the list
  ***********************************************************************************************/
  public void setValues(String[] valuesArray){
    this.valuesArray = valuesArray;
  }

}
