////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.component;

import org.watto.android.R;
import android.app.Activity;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.webkit.WebSettings;
import android.webkit.WebView;


public class WSWebView extends Activity {

  WebView webView;

  Thread loaderThread;


  public void onConfigurationChanged(Configuration configuration){
    super.onConfigurationChanged(configuration);
  }


  public void loadUrl(String url){
    if (loaderThread != null && loaderThread.isAlive()) {
      webView.stopLoading();
      loaderThread = null;
    }
    loaderThread = new Thread(new WSWebViewUrlLoader(webView,url));
    loaderThread.start();
  }

  public void postUrl (String url, byte[] postData){
    webView.postUrl(url,postData);
  }

  public void onCreate(Bundle savedInstanceState){
    super.onCreate(savedInstanceState);

    setContentView(R.layout.wswebview);
    webView = (WebView)findViewById(R.id.web_view);

    webView.setWebChromeClient(new WSWebChromeClient(this));
    webView.setWebViewClient(new WSWebViewClient(this));

    Intent intent = getIntent();
    String autoSize = intent.getStringExtra("autoSize");
    
    try {
      int width = getWindowManager().getDefaultDisplay().getWidth();
      int webpageWidth = 1024;
      int scalePercent = (int)(((double)width) / ((double)webpageWidth) * 100);

      if ((autoSize != null && autoSize.equals("false")) || scalePercent <= 0) {
        scalePercent = 1; // fit-to-screen
      }
      else if (scalePercent > 100) {
        scalePercent = 100;
      }
      webView.setInitialScale(scalePercent);
    }
    catch (Throwable t) {
      // Don't worry about resizing the WebView if it gets an error
    }

    WebSettings settings = webView.getSettings();
    settings.setJavaScriptEnabled(true);
    settings.setSupportZoom(true);
    settings.setBuiltInZoomControls(true);
    settings.setCacheMode(WebSettings.LOAD_NO_CACHE);
    settings.setLayoutAlgorithm(WebSettings.LayoutAlgorithm.NARROW_COLUMNS);
    settings.setLoadWithOverviewMode(true);

    // Get the URL that was set in the Intent
    String url = intent.getStringExtra("url");

    if (url == null || url.equals("")) {
      url = "about:blank";
    }

    // Load the URL in a separate Thread, to return control back to the user!
    // webView.loadUrl(url);
    loadUrl(url);

  }

}
