////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.component.popup;

import org.watto.android.DialogBundleManager;
import org.watto.android.event.WSViewClickableInterface;
import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.view.View;


/***********************************************************************************************
A popup <code>Dialog</code> that builds itself from an XML file and handles listener events
***********************************************************************************************/
public abstract class WSPopup extends Dialog implements WSViewClickableInterface {

  /** The Bundle that contains extra information for this Dialog **/
  protected Bundle bundle;


  /***********************************************************************************************
  Creates the popup and loads it from the XML file
  @param parent the <code>Context</code> that this popup belongs to
  @param id the Resource ID of the XML file used to build this popup
  ***********************************************************************************************/
  public WSPopup(Context parent, int id){
    super(parent);

    // Construct the Dialog from the XML file
    setContentView(id);

    // Register listeners against the dialog components
    registerListeners();
  }


  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    return false;
  }


  /***********************************************************************************************
  Prepares the components of this popup before it is shown. Calls <code>prepareDialog(Bundle)</code>
  with the <code>Bundle</code> from the <code>DialogBundleManager</code>
  ***********************************************************************************************/
  public final void prepareDialog(){
    bundle = DialogBundleManager.get();
    prepareDialog(bundle);
  }


  /***********************************************************************************************
  Prepares the components of this popup before it is shown
  @param bundle a <code>Bundle</code> containing extra information for this dialog
  ***********************************************************************************************/
  public abstract void prepareDialog(Bundle bundle);


  /***********************************************************************************************
  Registers the listeners on the components of this popup
  ***********************************************************************************************/
  public void registerListeners(){}

}
