////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.event.listener;

import android.view.View;
import android.view.View.OnClickListener;



/***********************************************************************************************
A single <code>OnClickListener</code> that triggers <i>onClick</i> events in multiple child
<code>OnClickListener</code>s. Allows multiple <code>OnClickListener</code>s to be registered on
a component that only allows 1 listener
***********************************************************************************************/
public class WSMultipleViewClickableListener implements OnClickListener {

  /** the listeners **/
  OnClickListener[] clickListeners;


  /***********************************************************************************************
  Constructor, without any initial <code>OnClickListener</code>s
  ***********************************************************************************************/
  public WSMultipleViewClickableListener(){
    this.clickListeners = new OnClickListener[0];
  }
  
  /***********************************************************************************************
  Creates a new listener that triggers <i>onClick</i> events in all the <i>listeners</i>
  @param listeners the <code>OnClickListener</code>s to register
  ***********************************************************************************************/
  public WSMultipleViewClickableListener(OnClickListener ... listeners){
    this.clickListeners = listeners;
  }


  /***********************************************************************************************
  Adds a number of <code>OnClickListener</code>s
  @param listeners the <code>OnClickListener</code>s to register
  ***********************************************************************************************/
  public void add(OnClickListener ... listeners){
    int newListenersCount = listeners.length;
      
    int oldLength = clickListeners.length;
    OnClickListener[] oldListeners = clickListeners;
    clickListeners = new OnClickListener[oldLength+newListenersCount];
    System.arraycopy(oldListeners,0,clickListeners,0,oldLength);
    System.arraycopy(listeners,0,clickListeners,oldLength,newListenersCount);
  }
    

  /***********************************************************************************************
  Calls <code>handler.onClick()</code> when a <i>single-click</i> event is triggered
  @param view the <code>View</code> that triggered the event
  ***********************************************************************************************/
  public void onClick(View view){
    int listenerCount = clickListeners.length;
    for (int i = 0;i < listenerCount;i++) {
      clickListeners[i].onClick(view);
    }
  }


}