////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.event.listener;

import org.watto.android.event.WSPreferenceChangableInterface;
import android.preference.Preference;


/***********************************************************************************************
Listens for <i>value-changed</i> events and passes them to the <code>WSPreferenceChangableInterface</code>
handler class for processing. These events are usually triggered by clicking on a <code>Preference<code>
***********************************************************************************************/
public class WSPreferenceChangableListener implements Preference.OnPreferenceChangeListener {

  /** the event handling class **/
  WSPreferenceChangableInterface handler;


  /***********************************************************************************************
  Registers the <code>WSPreferenceChangableInterface</code> handler class
  @param handler the event handling class
  ***********************************************************************************************/
  public WSPreferenceChangableListener(WSPreferenceChangableInterface handler){
    this.handler = handler;
  }


  /***********************************************************************************************
  Calls <code>handler.onChange()</code> when the value of a <code>Preference</code> is changed
  @param preference the <code>Preference</code> that triggered the event
  @param value the new value of the <code>Preference</code>
  @return <b>true</b> if the event was handled<br />
          <b>false</b> if the event wasn't handled.
  ***********************************************************************************************/
  public boolean onPreferenceChange(Preference preference, Object value){
    return handler.onChange(preference,value);
  }

}