////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.android.xml;

/***********************************************************************************************
A totally-new (NOT based of WSProgram 4.0!) XML Reader for Android that can also handle CDATA, 
comment, and declaration tags.
***********************************************************************************************/
public class XMLReader {

  protected static char[] textBuffer = new char[0];
  protected static int bufferPosition = 0;
  protected static int bufferLength = 0;
  /** The currently-read character **/
  protected static char character = ' ';

  //protected static String content = "";
  protected static StringBuffer contentBuffer;
  protected static boolean endOfBuffer = false;
  protected static boolean removeDuplicateWhitespace = true;
  protected static boolean lastCharacterWasWhitespace = false;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public XMLReader(){}


  /***********************************************************************************************
  Reads from the stream until a "]]>" tag is found
  ***********************************************************************************************/
  public static void findCDataTagFinish(boolean recordCharacters){
    removeDuplicateWhitespace = false;
    lastCharacterWasWhitespace = false;

    while (!endOfBuffer) {

      if (character == ']') {
        readChar();
        if (endOfBuffer) {
          return;
        }

        if (character == ']') {
          readChar();
          if (endOfBuffer) {
            return;
          }
          
          while (character == ']'){
            // Accounts for the ']' character appearing at least once right before the end tag
            contentBuffer.append(']');
            readChar();
          }

          if (character == '>') {
            // Found the full "]]>" tag
            
            // Finally, skip over the '>' character so we don't add it to the content in the next step
            readChar();
            
            return;
          }
          else {
            if (recordCharacters) {
              //content += "]]";
              contentBuffer.append("]]");
            }
            moveCharacterToContent(recordCharacters);
          }

        }
        else {
          if (recordCharacters) {
            //content += "]";
            contentBuffer.append("]");
          }
          moveCharacterToContent(recordCharacters);
        }

      }
      else {
        moveCharacterToContent(recordCharacters);
      }

      readChar();
      //content += findAndMoveTo(']',true);
      
      if (endOfBuffer) {
        return;
      }
    }
  }


  /***********************************************************************************************
  Reads from the stream until a "-->" tag is found
  ***********************************************************************************************/
  public static void findCommentTagFinish(boolean recordCharacters){
    removeDuplicateWhitespace = true;
    lastCharacterWasWhitespace = false;

    while (!endOfBuffer) {

      if (character == '-') {
        readChar();
        if (endOfBuffer) {
          return;
        }

        if (character == '-') {
          readChar();
          if (endOfBuffer) {
            return;
          }

          if (character == '>') {
            // Found the full "-->" tag
            
            // Finally, skip over the '>' character so we don't add it to the content in the next step
            readChar();
            
            return;
          }
          else {
            if (recordCharacters) {
              //content += "--";
              contentBuffer.append("--");
            }
            moveCharacterToContent(recordCharacters);
          }

        }
        else {
          if (recordCharacters) {
            //content += "-";
            contentBuffer.append("-");
          }
          moveCharacterToContent(recordCharacters);
        }

      }
      else {
        moveCharacterToContent(recordCharacters);
      }

      readChar();
      if (endOfBuffer) {
        return;
      }
    }
  }


  /***********************************************************************************************
  Reads from the stream until a "?>" tag is found
  ***********************************************************************************************/
  public static void findDeclarationTagFinish(boolean recordCharacters){
    removeDuplicateWhitespace = true;
    lastCharacterWasWhitespace = false;

    while (!endOfBuffer) {

      if (character == '?') {
        readChar();
        if (endOfBuffer) {
          return;
        }

        if (character == '>') {
          // Found the full "?>" tag
          
          // Finally, skip over the '>' character so we don't add it to the content in the next step
          readChar();
          
          return;
        }
        else {
          if (recordCharacters) {
            //content += "?";
            contentBuffer.append("?");
          }
          moveCharacterToContent(recordCharacters);
        }

      }
      else {
        moveCharacterToContent(recordCharacters);
      }

      readChar();
      if (endOfBuffer) {
        return;
      }
    }
  }


  /***********************************************************************************************
  Reads from the stream until a '>' tag is found
  ***********************************************************************************************/
  public static void findTagFinish(boolean recordCharacters){
    removeDuplicateWhitespace = true;
    lastCharacterWasWhitespace = false;

    while (character != '>') {
      moveCharacterToContent(recordCharacters);

      readChar();
      if (endOfBuffer) {
        return;
      }
    }
    
    //content += findAndMoveTo('>',true);


    // Finally, skip over the '>' character so we don't add it to the content in the next step
    readChar();
  }


  /***********************************************************************************************
  Reads from the stream until a '<' tag is found
  ***********************************************************************************************/
  public static void findTagStart(boolean recordCharacters){
    removeDuplicateWhitespace = true;
    lastCharacterWasWhitespace = false;

    while (character != '<') {
      moveCharacterToContent(recordCharacters);

      readChar();
      if (endOfBuffer) {
        return;
      }
    }
  }


  /***********************************************************************************************
  Reads from the stream until a '<' tag is found
  ***********************************************************************************************/
  public static void moveCharacterToContent(boolean recordCharacters){
    // don't move it, because we don't want to record it
    if (!recordCharacters) {
      return;
    }

    // Move it, because we want to keep the content exactly as it is
    if (!removeDuplicateWhitespace) {
      //content += character;
      contentBuffer.append(character);
      return;
    }

    // We want to remove duplicate whitespace
    if (character == '\n' || character == '\t' || character == '\r') {
      character = ' ';
    }
    if (character == ' ') {
      // Don't record the whitespace if the content is empty already - don't want
      // to start with a whitespace character in the content
      //if (content.length() == 0) {
      if (contentBuffer.length() == 0){
        return;
      }

      // found a whitespace - see whether it is repeating or not
      if (lastCharacterWasWhitespace) {
        // ignore duplicate whitespace
        return;
      }
      else {
        // not a duplicate, but prepare to allow duplicates
        //content += character;
        contentBuffer.append(character);
        lastCharacterWasWhitespace = true;
      }
    }
    else {
      // not whitespace
      //content += character;
      contentBuffer.append(character);
      lastCharacterWasWhitespace = false;
    }

  }


  /***********************************************************************************************
  Reads an XML-format <code>String</code> into a tree of <code>XMLNode</code>s
  @param text the XML-format <code>String</code>
  @return the tree of <code>XMLNode</code>s representing the XML-format <code>String</code>
  ***********************************************************************************************/
  public static XMLNode read(String text){
    XMLNode root = new XMLNode();
    read(text,root);
    try {
      return (XMLNode)root.getChild(0);
    }
    catch (Throwable t) {
      return root;
    }
  }


  /***********************************************************************************************
  Reads an XML-format <code>String</code> into a tree of <code>XMLNode</code>s. The tree is
  constructed using the given <code>root</code> node
  @param text the XML-format <code>String</code>
  @param root the root node of the tree, into which the tree will be constructed
  ***********************************************************************************************/
  public static void read(String text,XMLNode root){
    textBuffer = text.toCharArray();
    bufferLength = textBuffer.length;
    bufferPosition = 0;
    
    endOfBuffer = false;
    
    lastCharacterWasWhitespace = false;
    removeDuplicateWhitespace = true;
    
    //content = "";
    contentBuffer = new StringBuffer();
    
    character = ' ';

    readTag(root);
  }


  /***********************************************************************************************
  Reads the next <code>character</code> from the <code>manipulator</code>
  ***********************************************************************************************/
  public static void readChar(){
    try {
      character = textBuffer[bufferPosition];
      bufferPosition++;

      if (bufferPosition == bufferLength) {
        endOfBuffer = true;
      }
      else {
        endOfBuffer = false;
      }
    }
    catch (Throwable t) {
      character = Character.UNASSIGNED;
      endOfBuffer = true;
    }
  }
  
  
  /***********************************************************************************************
  Finds the next occurance of <i>findCharacter</i> and moves the buffer to this point, returning 
  a String of all the data from the current position to the new position
  @param findCharacter the character to search for
  @return a String of all read data from the current buffer position to the new position
  ***********************************************************************************************/
  public static String findAndMoveTo(char findCharacter, boolean returnResult){
    int startBufferPosition = bufferPosition - 1;
    if (startBufferPosition == -1){
      startBufferPosition = 0;
    }
    
    
    boolean found = false;
    while (bufferPosition < bufferLength) {
      if (textBuffer[bufferPosition] == findCharacter){
        found = true;
        character = textBuffer[bufferPosition]; // set the character so other methods can use it
        break;
      }
      bufferPosition++;
    }
    
    if (bufferPosition >= bufferLength){
      endOfBuffer = true;
    }
    else {
      endOfBuffer = false;
    }
    
    if (!found){
      return null;
    }
    
    if (returnResult){
      return new String(textBuffer,startBufferPosition,bufferPosition-startBufferPosition);
    }
    else {
      return "";
    }

  }


  /***********************************************************************************************
  Reads a single tag from the current position in the <code>manipulator</code>, adding it as a 
  child to the <code>root</code> node
  @param root the root node to add this new tag to
  ***********************************************************************************************/
  public static void readTag(XMLNode root){
    try {

      //System.out.println("Tag");

      // Continue looping over until all the buffer has been read
      while (!endOfBuffer) {

        // Find the start of the next tag, recording all characters
        findTagStart(true);

        // If there was any content, set it on this node
        //if (content.length() > 0) {
        if (contentBuffer.length() > 0){
          // Set the content of the current tag
          //root.setContent(content);
          root.setContent(contentBuffer.toString());
          // Reset the content
          //content = "";
          contentBuffer = new StringBuffer();
        }

        if (endOfBuffer) {
          return;
        }

        // Skip over the '<' tag and find the next character
        readChar();

        if (endOfBuffer) {
          return;
        }

        if (character == '/') {
          // Reading a closing tag
          findTagFinish(false);
          // Finished reading this tag - it has been closed
          return;
        }
        else if (character == '!') {
          // Read the next character to determine what exactly this content is
          readChar();

          if (endOfBuffer) {
            return;
          }

          if (character == '[') {
            // Skip over the CData start tag
            skipCDataTagStart();
            // Find the end of the CData, recording all characters as we go
            findCDataTagFinish(true);
            // Set the content of the current tag to the CData-read content
            //root.setContent(content);
            root.setContent(contentBuffer.toString());
            // Reset the content
            //content = "";
            contentBuffer = new StringBuffer();
          }
          else {
            // skip over the comment (find the comment finish tag)
            findCommentTagFinish(false);
            // Finished reading this tag - it was a comment
            //return;
          }
        }
        else if (character == '?') {
          // skip over the declaration (find the declaration finish tag)
          findDeclarationTagFinish(false);
          // Finished reading this tag - it was a declaration
          //return;
        }
        else {
          // Find the end of the normal tag, recording all characters as we go
          findTagFinish(true);

          // Determine whether this is a single tag
          boolean singleTag = false;
          //int contentLength = content.length();
          int contentLength = contentBuffer.length();
          if (contentLength > 0) {
            //char lastCharacter = content.charAt(contentLength - 1);
            char lastCharacter = contentBuffer.charAt(contentLength - 1);
            if (lastCharacter == '/') {
              // It is a single tag
              singleTag = true;
              // Strip off the slash
              //content = content.substring(0,contentLength - 1);
              contentBuffer.deleteCharAt(contentLength-1);
            }
          }

          // trim any leading and trailing whitespace from the tag
          //content = content.trim();
          contentBuffer = new StringBuffer(contentBuffer.toString().trim());
          // Create the node
          // Determine whether there are attributes in the tag or not
          XMLNode node = null;
          //int whiteSpacePosition = content.indexOf(' ');
          int whiteSpacePosition = contentBuffer.indexOf(" ");
          if (whiteSpacePosition > 0) {
            // Create the node with only the tag name
            //node = new XMLNode(content.substring(0,whiteSpacePosition));
            node = new XMLNode(contentBuffer.substring(0,whiteSpacePosition));
            // Change the content to only be attributes
            //content = content.substring(whiteSpacePosition + 1);
            String contentString = contentBuffer.substring(whiteSpacePosition + 1);
            // Set the attributes on the node
            //splitAttributes(content,node);
            splitAttributes(contentString,node);
          }
          else {
            // No attributes found, so just create the node
            //node = new XMLNode(content);
            node = new XMLNode(contentBuffer.toString());
          }

          // Reset the content
          //content = "";
          contentBuffer = new StringBuffer();

          // Add the tag node to the root
          root.addChild(node);

          // Read the contents of this new tag node (if its not a single tag)
          if (!singleTag) {
            readTag(node);
          }
        }

        // We're done reading this tag - lets loop back and look for the next one!
      }

    }
    catch (Throwable t) {
      //System.out.println("error");
    }
  }


  /***********************************************************************************************
  Skips over the CData header in the buffer
  ***********************************************************************************************/
  public static void skipCDataTagStart(){
    removeDuplicateWhitespace = true;
    lastCharacterWasWhitespace = false;

    if (character == '[') {
      readChar();
      if (endOfBuffer) {
        return;
      }

      if (character == 'C') {
        readChar();
        if (endOfBuffer) {
          return;
        }

        if (character == 'D') {
          readChar();
          if (endOfBuffer) {
            return;
          }

          if (character == 'A') {
            readChar();
            if (endOfBuffer) {
              return;
            }

            if (character == 'T') {
              readChar();
              if (endOfBuffer) {
                return;
              }

              if (character == 'A') {
                readChar();
                if (endOfBuffer) {
                  return;
                }

                if (character == '[') {
                  // Found the full "<![CDATA[" tag
                  
                  // Finally, skip over the '[' character so we don't add it to the content in the next step
                  readChar();
                  
                  return;
                }
                else {
                  //content += "[CDATA";
                  contentBuffer.append("[CDATA");
                  moveCharacterToContent(true);
                }

              }
              else {
                //content += "[CDAT";
                contentBuffer.append("[CDAT");
                moveCharacterToContent(true);
              }

            }
            else {
              //content += "[CDA";
              contentBuffer.append("[CDA");
              moveCharacterToContent(true);
            }

          }
          else {
            //content += "[CD";
            contentBuffer.append("[CD");
            moveCharacterToContent(true);
          }
        }
        else {
          //content += "[C";
          contentBuffer.append("[C");
          moveCharacterToContent(true);
        }

      }
      else {
        //content += "[";
        contentBuffer.append("[");
        moveCharacterToContent(true);
      }

    }
    else {
      moveCharacterToContent(true);
    }

  }


  /***********************************************************************************************
  Reads the attributes of the tag into the <code>root</code> node
  @param currentNode the tag that is having its attributes read.
  ***********************************************************************************************/
  public static void splitAttributes(String content,XMLNode node){
    try {

      //System.out.println("attributes");

      String[] blocks = content.split("\"");
      int blocksCount = blocks.length;
      for (int i = 0;i < blocksCount;i += 2) {
        // Grab the attribute and value pair
        String attribute = blocks[i].trim();
        String value = blocks[i + 1].trim();

        // strip the '=' sign off the end of the attribute
        attribute = attribute.substring(0,attribute.length() - 1).trim();

        // record the attribute
        node.addAttribute(attribute,value);
      }

    }
    catch (Throwable t) {}
  }

}